package com.qmth.paper.library.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.paper.library.business.entity.PaperArchives;
import com.qmth.paper.library.business.entity.PaperScanTask;
import com.qmth.paper.library.business.service.BusinessCacheService;
import com.qmth.paper.library.business.service.PaperArchivesService;
import com.qmth.paper.library.business.service.PaperLibraryService;
import com.qmth.paper.library.business.service.PaperScanTaskService;
import com.qmth.paper.library.common.bean.dto.SchoolDto;
import com.qmth.paper.library.common.bean.dto.syssetting.SettingObject;
import com.qmth.paper.library.common.bean.dto.syssetting.SimpleObject;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.entity.*;
import com.qmth.paper.library.common.enums.ExceptionResultEnum;
import com.qmth.paper.library.common.lock.LockService;
import com.qmth.paper.library.common.lock.LockType;
import com.qmth.paper.library.common.service.*;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.CachePut;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class BusinessCacheServiceImpl implements BusinessCacheService {

    Map<Long, Integer> scanTaskSequenceMap = new HashMap<>();
    Map<Long, Integer> paperArchivesCodeMap = new HashMap<>();
    Map<Long, Integer> scanTaskCodeMap = new HashMap<>();

    @Resource
    PaperScanTaskService paperScanTaskService;

    @Resource
    PaperArchivesService paperArchivesService;

    @Resource
    PaperLibraryService paperLibraryService;

    @Resource
    SysUserService sysUserService;

    @Resource
    BasicSchoolService basicSchoolService;

    @Resource
    LockService lockService;

    @Override
    @Cacheable(value = SystemConstant.CACHE_NAME, key = "'paperScanTask' + #scanTaskId")
    public PaperScanTask getPaperScanTask(Long scanTaskId) {
        return paperScanTaskService.getById(scanTaskId);
    }

    @Override
    @CacheEvict(value = SystemConstant.CACHE_NAME, key = "'paperScanTask' + #scanTaskId")
    public void removeSysSetting(Long schoolId) {

    }

    @Override
    @Cacheable(value = SystemConstant.CACHE_NAME, key = "'sysUser' + #userId")
    public SysUser getSysUser(Long userId) {
        return sysUserService.getById(userId);
    }

    @Override
    public int getSequence(Long paperScanTaskId) {
        try {
            lockService.waitlock(LockType.GET_SEQUENCE, paperScanTaskId);
            if (!scanTaskSequenceMap.containsKey(paperScanTaskId)) {
                scanTaskSequenceMap.put(paperScanTaskId, 1);
                return 1;
            }
            Integer integer = scanTaskSequenceMap.get(paperScanTaskId);
            int value = integer + 1;
            scanTaskSequenceMap.put(paperScanTaskId, value);
            return value;
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("获取自增数失败：" + e.getMessage());
        } finally {
            lockService.unlock(LockType.GET_SEQUENCE, paperScanTaskId);
        }
    }

    @Override
    public void initCacheData() {
        QueryWrapper<PaperScanTask> queryWrapper = new QueryWrapper<>();
        List<PaperScanTask> paperScanTaskList = paperScanTaskService.list(queryWrapper);
        for (PaperScanTask paperScanTask : paperScanTaskList) {
            Integer sequence = paperLibraryService.maxSequence(paperScanTask.getId());
            int value = sequence == null ? 0 : sequence.intValue();
            scanTaskSequenceMap.put(paperScanTask.getId(), value);
        }

        List<SchoolDto> basicSchoolList = basicSchoolService.listSchool();
        for (SchoolDto schoolDto : basicSchoolList) {
            Long schoolId = Long.valueOf(schoolDto.getId());
            // 档案编号序号
            QueryWrapper<PaperArchives> archivesQueryWrapper = new QueryWrapper<>();
            archivesQueryWrapper.lambda().eq(PaperArchives::getSchoolId, schoolId)
                    .likeRight(PaperArchives::getArchivesCode, SystemConstant.PAPER_ARCHIVES_CODE_PREFIX)
                    .orderByDesc(PaperArchives::getArchivesCode);
            List<PaperArchives> paperArchivesList = paperArchivesService.list(archivesQueryWrapper);
            if (!CollectionUtils.isEmpty(paperArchivesList)) {
                String paperArchivesCode = paperArchivesList.get(0).getArchivesCode();
                String codeTemp = paperArchivesCode.substring(SystemConstant.PAPER_ARCHIVES_CODE_PREFIX.length());
                int codeInt = Integer.parseInt(codeTemp);
                paperArchivesCodeMap.put(schoolId, codeInt);
            }

            // 扫描任务编号序号
            QueryWrapper<PaperScanTask> scanTaskQueryWrapper = new QueryWrapper<>();
            scanTaskQueryWrapper.lambda().eq(PaperScanTask::getSchoolId, schoolId)
                    .likeRight(PaperScanTask::getScanTaskCode, SystemConstant.SCAN_TASK_CODE_PREFIX)
                    .orderByDesc(PaperScanTask::getScanTaskCode);
            List<PaperScanTask> scanTaskList = paperScanTaskService.list(scanTaskQueryWrapper);
            if (!CollectionUtils.isEmpty(scanTaskList)) {
                String scanTaskCode = scanTaskList.get(0).getScanTaskCode();
                String codeTemp = scanTaskCode.substring(SystemConstant.SCAN_TASK_CODE_PREFIX.length());
                int codeInt = Integer.parseInt(codeTemp);
                scanTaskCodeMap.put(schoolId, codeInt);
            }
        }
    }

    @Override
    public int getPaperArchivesCodeSequence(Long schoolId) {
        try {
            lockService.waitlock(LockType.PAPER_ARCHIVES_SEQUENCE, schoolId);
            if (!paperArchivesCodeMap.containsKey(schoolId)) {
                paperArchivesCodeMap.put(schoolId, 1);
                return 1;
            }
            Integer integer = paperArchivesCodeMap.get(schoolId);
            int value = integer + 1;
            paperArchivesCodeMap.put(schoolId, value);
            return value;
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("获取档案编号自增数失败：" + e.getMessage());
        } finally {
            lockService.unlock(LockType.PAPER_ARCHIVES_SEQUENCE, schoolId);
        }
    }

    @Override
    public int getScanTaskCodeSequence(Long schoolId) {
        try {
            lockService.waitlock(LockType.SCAN_TASK_SEQUENCE, schoolId);
            if (!scanTaskCodeMap.containsKey(schoolId)) {
                scanTaskCodeMap.put(schoolId, 1);
                return 1;
            }
            Integer integer = scanTaskCodeMap.get(schoolId);
            int value = integer + 1;
            scanTaskCodeMap.put(schoolId, value);
            return value;
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("获取扫描任务编号自增数失败：" + e.getMessage());
        } finally {
            lockService.unlock(LockType.SCAN_TASK_SEQUENCE, schoolId);
        }
    }
}
